@@ -18,10 +18,10 @@
 
 
 #include <math.h>
-#include "R.h"
+#include <R.h>
 
 
-extern void F77_SYMBOL(muin2ser) ();
+extern void F77_NAME(muin2ser) ();
 
 void R_amif (double *x, int *lx, double *inf, int *k, int *maxbit, 
 	     double *confidence, int *norm, int *trace)
@@ -50,7 +50,7 @@ void R_amif (double *x, int *lx, double *inf, int *k, int *maxbit,
     {
       x0[j] = x[j]; y0[j] = x[j+i];
     }
-    F77_SYMBOL(muin2ser) (&information, lx, &freq, x0, y0, intx, inty, 
+    F77_CALL(muin2ser) (&information, lx, &freq, x0, y0, intx, inty, 
 			  s, q, q_unsort, indices_x, indices_y, 
 			  position, maxbit, confidence, trace);
     if (*norm) inf[i] = sqrt(1.0-exp((-2.0)*information));
